# Python imports

# Lib imports
import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Gtk
from gi.repository import Gdk

# Application imports
from ..widgets.image_view import ImageView



class ImageViewEveBox(Gtk.EventBox):
    def __init__(self, vadjustment, hadjustment):
        super(ImageViewEveBox, self).__init__()

        self.vadjustment   = vadjustment
        self.hadjustment   = hadjustment
        self._is_dragging  = False
        self._drag_start_x = 0
        self._drag_start_y = 0
        self._current_x    = 0
        self._current_y    = 0

        self._setup_styling()
        self._setup_signals()
        self._subscribe_to_events()
        self._load_widgets()

        self.show_all()


    def _setup_styling(self):
        ...

    def _setup_signals(self):
        self.connect("button-press-event", self._press_event)
        self.connect("motion-notify-event", self._move_motion_event)
        self.connect("button-release-event", self._release_event)

    def _subscribe_to_events(self):
        ...

    def _load_widgets(self):
        self.add(ImageView())

    def _press_event(self, widget = None, eve = None):
        self.set_cursor(Gdk.CursorType.CROSSHAIR)

        self._is_dragging  = True
        self._drag_start_x = eve.x_root
        self._drag_start_y = eve.y_root

    def _move_motion_event(self, widget = None, eve = None):
        if self._is_dragging:
            vval      = self.vadjustment.get_value()
            hval      = self.hadjustment.get_value()
            hstep_val = self.hadjustment.get_step_increment()
            vstep_val = self.vadjustment.get_step_increment()

            if eve.x_root > self._drag_start_x:
                self.hadjustment.set_value(hval + hstep_val)
            elif eve.x_root < self._drag_start_x:
                self.hadjustment.set_value(hval - hstep_val)
            else:
                self.hadjustment.set_value(hval)

            if eve.y_root > self._drag_start_y:
                self.vadjustment.set_value(vval + vstep_val)
            elif eve.y_root < self._drag_start_y:
                self.vadjustment.set_value(vval - vstep_val)
            else:
                self.vadjustment.set_value(vval)

            self._drag_start_x = eve.x_root
            self._drag_start_y = eve.y_root


    def _release_event(self, widget = None, eve = None):
        self.set_cursor(Gdk.CursorType.ARROW)

        self._is_dragging  = False
        self._drag_start_x = 0
        self._drag_start_y = 0

    def set_cursor(self, type = None):
        window = settings.get_main_window()
        cursor = Gdk.Cursor(type)
        window.get_window().set_cursor(cursor)
